@@ -136,6 +136,7 @@ class Attention(nn.Module):
         self,
         dim,
         *,
+        context_dim = None,
         dim_head = 64,
         heads = 8,
         causal = False,
@@ -143,6 +144,8 @@ class Attention(nn.Module):
         null_kv = False
     ):
         super().__init__()
+        context_dim = default(context_dim, dim)
+
         self.heads = heads
         self.scale = dim_head ** -0.5
         self.causal = causal
@@ -151,8 +154,8 @@ class Attention(nn.Module):
         self.dropout = nn.Dropout(dropout)
 
         self.to_q = nn.Linear(dim, inner_dim, bias = False)
-        self.to_k = nn.Linear(dim, inner_dim, bias = False)
-        self.to_v = nn.Linear(dim, inner_dim, bias = False)
+        self.to_k = nn.Linear(context_dim, inner_dim, bias = False)
+        self.to_v = nn.Linear(context_dim, inner_dim, bias = False)
         self.to_out = nn.Linear(inner_dim, dim)
 
         # allowing for attending to nothing (null function)
@@ -302,6 +305,7 @@ class Encoder(nn.Module):
         dim,
         *,
         depth,
+        context_dim = None,
         causal = False,
         heads = 8,
         dim_head = 64,
@@ -311,6 +315,7 @@ class Encoder(nn.Module):
         final_norm = True,
         cross_attn_layers = None,
         post_norm = False,
+        output_dim = None,
         norm_klass = RMSNorm,
         scale_residual = 1.
     ):
@@ -327,11 +332,12 @@ class Encoder(nn.Module):
 
             self.layers.append(nn.ModuleList([
                 wrapper(Attention(dim = dim, dim_head = dim_head, heads = heads, dropout = attn_dropout, causal = causal)),
-                wrapper(Attention(dim = dim, dim_head = dim_head, heads = heads, dropout = attn_dropout)) if has_cross_attn else None,
+                wrapper(Attention(dim = dim, context_dim = context_dim, dim_head = dim_head, heads = heads, dropout = attn_dropout)) if has_cross_attn else None,
                 wrapper(FeedForward(dim = dim, mult = ff_mult, dropout = ff_dropout)),
             ]))
 
         self.norm_out = norm_klass(dim) if final_norm and not post_norm else nn.Identity()
+        self.project_out = nn.Linear(dim, output_dim) if exists(output_dim) else nn.Identity()
 
     def forward(self, x, *, mask = None, chunked_seq):
         device, chunk_size, seq_len = x.device, x.shape[-2], chunked_seq.shape[-2]
@@ -347,7 +353,8 @@ class Encoder(nn.Module):
 
             x = ff(x)
 
-        return self.norm_out(x)
+        x = self.norm_out(x)
+        return self.project_out(x)
 
 class Decoder(nn.Module):
     def __init__(
@@ -388,10 +395,17 @@ class Decoder(nn.Module):
 
         self.norm_out = norm_klass(dim) if final_norm and not post_norm else nn.Identity()
 
-    def forward(self, x, *, context_mask = None, retrieved = None):
+    def forward(self, x, encoder, *, encoder_retrieved_mask = None, context_mask = None, retrieved = None):
         device, seq_len = x.device, x.shape[-2]
         self_attn_pos_emb = self.rotary_pos_emb(seq_len, device = device)
 
+        # calculate seq index
+
+        num_seq_chunks = seq_len // self.chunk_size
+        seq_index = num_seq_chunks * self.chunk_size
+
+        # rotary positions on the retrieved chunks
+
         if exists(retrieved):
             num_chunks, num_neighbors, chunk_size = retrieved.shape[-4:-1]
 
@@ -400,10 +414,24 @@ class Decoder(nn.Module):
 
             cross_attn_pos_emb = (cross_attn_q_pos_emb, cross_attn_k_pos_emb)
 
+        # keep track of whether retrieved tokens are encoded yet
+
+        retrieved_encoded = False
+
+        # go through the decoder layers
+
         for attn, cross_attn, ff in self.layers:
             x = attn(x, pos_emb = self_attn_pos_emb)
 
             if exists(cross_attn) and exists(retrieved):
+                if not retrieved_encoded:
+                    retrieved = rearrange(retrieved, 'b k r n d -> (b k r) n d')
+                    seq_as_context = repeat(x[:, :seq_index], 'b (k n) d -> (b k r) n d', n = self.chunk_size, r = num_neighbors)
+
+                    retrieved = encoder(retrieved, mask = encoder_retrieved_mask, chunked_seq = seq_as_context)
+                    retrieved = rearrange(retrieved, '(b k r) n d -> b k r n d', k = num_chunks, r = num_neighbors)
+                    retrieved_encoded = True
+
                 x = cross_attn(
                     x,
                     context = retrieved,
@@ -473,13 +501,15 @@ class RETRO(nn.Module):
 
         self.encoder = Encoder(
             dim = enc_dim,
+            context_dim = dec_dim,
             depth = enc_depth,
             attn_dropout = enc_attn_dropout,
             ff_dropout = enc_ff_dropout,
             cross_attn_layers = enc_cross_attn_layers,
             post_norm = use_deepnet,
             norm_klass = norm_klass,
-            scale_residual = enc_scale_residual
+            scale_residual = enc_scale_residual,
+            output_dim = dec_dim
         )
 
         self.decoder = Decoder(
@@ -590,22 +620,19 @@ class RETRO(nn.Module):
             encoder_retrieved_mask = rearrange(mask, 'b k r n -> (b k r) n')
             decoder_retrieved_mask = mask
 
-        # encode
-
-        retrieved = rearrange(retrieved, 'b k r n d -> (b k r) n d')
-        embed_as_context = repeat(embed[:, :seq_index], 'b (k n) d -> (b k r) n d', n = self.chunk_size, r = num_neighbors)
-
-        retrieved = self.encoder(retrieved, mask = encoder_retrieved_mask, chunked_seq = embed_as_context)
-        retrieved = rearrange(retrieved, '(b k r) n d -> b k r n d', k = num_chunks, r = num_neighbors)
-
         # project both sequence embedding and retrieved embedding to decoder dimension if necessary
 
         embed = self.to_decoder_model_dim(embed)
-        retrieved = self.encoder_output_to_decoder_dim(retrieved)
 
         # decode
 
-        embed = self.decoder(embed, context_mask = decoder_retrieved_mask, retrieved = retrieved)
+        embed = self.decoder(
+            embed,
+            self.encoder,
+            context_mask = decoder_retrieved_mask,
+            encoder_retrieved_mask = encoder_retrieved_mask,
+            retrieved = retrieved
+        )
 
         # project to logits
 
